//////////////////////////////////////////////////////////////////
// HTTP-Server mit Atmeg8 und XPort
// Zielhardware: Atmel AtMega 8
// Dateiname:    main.c
// Headerfile:   main.h
// Autor: Thomas Wiens th.wiens(at)gmx.de
// Datum: 9.6.2006
//////////////////////////////////////////////////////////////////

#include "main.h"

void send_prog_data(char *text);
char parse_line(char *line);
// ********************************************************************************************
// GLOBALE VARIABLEN
// ********************************************************************************************
char buffer[100];

volatile uint8_t buf_index = 0;
volatile uint8_t recv_state = 0;

const uint8_t index_html[] PROGMEM=
#include "index.html"

const uint8_t test_html[] PROGMEM=
#include "test.html"

const uint8_t fehler_html[] PROGMEM=
#include "fehler.html"

const uint8_t status_html[] PROGMEM=
#include "status.html"

// ********************************************************************************************
// INTERRUPTS
// ********************************************************************************************
SIGNAL (SIG_UART_RECV) {    
    char zeichen;
    zeichen = UDR;
    // State-Machine
    switch (recv_state){
        case RECV_NONE:                                     // Noch nichts empfangen
            if (zeichen == 'G')
                recv_state = RECV_G;
            else
                recv_state = RECV_NONE;
            break;        
        case RECV_G:
            if (zeichen == 'E')
                recv_state = RECV_E;
            else
                recv_state = RECV_NONE;
            break; 
        case RECV_E:
            if (zeichen == 'T')
                recv_state = RECV_T;
            else
                recv_state = RECV_NONE;
            break;   
        case RECV_T:                                        // GET eingelesen -> Rest zur Auswertung in den Puffer
            if (zeichen == CR)                              // Zeilenende durch LF angezeigt
                recv_state = RECV_GET_LINE;
            else
                buffer[buf_index++] = zeichen;              // GET Zeile einlesen
            break;
        case RECV_GET_LINE:                                 // Auf abschlieende Leerzeile warten 
            if (zeichen == CR)                              // CR erhalten
                recv_state = RECV_CR1;                
            break;
        case RECV_CR1: 
            if (zeichen == LF)                              // nach CR ein LF erhalten
                recv_state = RECV_LF1;
            else
                recv_state = RECV_GET_LINE;                  // anderes Zeichen erhalten > reset
            break;
        case RECV_LF1: 
            if (zeichen == CR)                              // nach CR und LF ein CR erhalten
                recv_state = RECV_CR2;
            else
                recv_state = RECV_GET_LINE;                  // anderes Zeichen erhalten > reset
            break;
        case RECV_CR2: 
            if (zeichen == LF){                             // nach CR->LF->CR ein LF erhalten -> Leerzeile
                recv_state = RECV_SPARE_ROW;
                buffer[++buf_index] = '\0';
            }
            else
                recv_state = RECV_GET_LINE;                  // anderes Zeichen erhalten > reset       
    }
    
    if (buf_index >= 99) {                                  // Buffer ist voll, alles resetten
        buffer[0] = '\0';
        buf_index = 0;
        recv_state = RECV_NONE;
    }    
}

// ********************************************************************************************
// MAIN
// ********************************************************************************************
int main(void){  
    char page;
	uinit();										// UART initialisieren
	UCSRB |= (1 << RXCIE);							// Receive Interrupt freigeben
	sei();
    
	for (;;){
		if (recv_state == RECV_SPARE_ROW) {
            cli();
            page = parse_line(buffer);
            switch(page){
                case 'I':
                    send_prog_data(index_html);
                    break;
                case 'T':
                    send_prog_data(test_html);
                    break;   
                case 'F':
                    send_prog_data(fehler_html);
                    break; 
                case 'S':
                    send_prog_data(status_html);
                    break;            
            }        
            buf_index = 0;
            buffer[0] = '\0';
            recv_state = RECV_NONE;
            sei();
        }		
	}	
	return(0);
}
// ********************************************************************************************
// FUNCTIONS
// ********************************************************************************************
void send_prog_data(char *text){
    uint8_t zeichen;
    int i;
    for (i = 0; i < strlen_P(text); i++) {
        zeichen = pgm_read_byte(&(text[i]));
        if (zeichen == '$') {                       // Wert einfgen
            i++;                                    // Nachfolgendes Zeichen holen
            zeichen = pgm_read_byte(&(text[i]));
            // Wird in der HTML-Datei ein $A eingetragen, so kann es hier durch 
            // entsprechende Werte (z.B. A/D Werte usw.) dynamisch bei jeder Anfrage
            // ersetzt werden
            switch (zeichen) {
                case 'A':
                    uputs("1234");
                    break;
                case 'B':
                    uputs("567");
                    break;
            }
        }
        else 
            uputchar(zeichen);
    }
}

char parse_line(char *line) {
    char filename[80];  
    char *pos_1, *pos_2;
    int len; 
    pos_1 = strstr(line, " /");
    pos_2 = strstr(line, "HTTP");    
    if (!pos_2)
       return ('F');                            // "HTTP" nicht in der Zeile vorhanden    
    // Lnge des Dateinamens
    pos_1 += 2;
    pos_2 -= 1;
    len = pos_2 - pos_1;
    if (len < 0)
         return ('F');                            // Falsche Reihenfolge -> Fehlerhafte Anfrage
    
    strncpy(filename, pos_1, len);
    filename[len] = 0;
    
    if (strstr(filename, "index.html"))
        return ('I');
    else if (strstr(filename, "test.html"))
        return ('T');
    else if (strstr(filename, "status.html"))
        return ('S');
    else return ('F');                           // Datei ist nicht vorhanden
}


